#ifndef __ATARI_LEDS_H__
#define __ATARI_LEDS_H__

#pragma once
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
  ATARI_LEDS_UNKNOWN = 0,
  ATARI_LEDS_CLASSIC,
  ATARI_LEDS_MODERN
} AtariDeviceType;

struct atari_leds;

/* Atari Leds Libary - Small helper library for atari controller leds
 *
 * Leds on the atari controllers are normally under firmware control; However
 * they can be controlled by userspace applications. This libary provides some
 * helpful functions to do just that.
 *
 * The expected model is roughly:
 * // Allocate/Open the device
 * struct atari_leds *l = atari_leds_new_from_input ("/dev/input/eventX");
 *
 * // Prepare led configuration
 * atari_leds_clear (l);
 * atari_leds_set (l, 0, 255);
 * atari_leds_set_block (l, 8, 4, 128);
 * // Commit leds to write out to device
 * atari_leds_commit (l);
 *
 * // When done let the firmware control the leds again and free resources
 * atari_leds_drop_control (leds);
 * atari_leds_free (leds);
 *
 *
 * The control of the leds will move from the controller firmware to the
 * application once the first _commit call has been done and can be moved back
 * at any time explicitly when calling atari_leds_drop_control.
 *
 * Note that it is assume only one application at a time is controlling the
 * LEDs. No effort is done support multiple applications simultaneously
 * twiddling the led controls
 *
 * To build applications using this library; Use the atari-leds pkg-config file
 * in your build system of choice
 */

/*
 * Open an Atari device led device; should be freed with atari_leds_free.
 *  - devpath: Path to input device for the controller (e.g.  /dev/input/event3)
 *
 * Returns NULL on failure or a newly allocate struct otherwise
 */
struct atari_leds *atari_leds_new_from_input (const char *devpath);

/*
 * Free atari_leds structure and associated resources
 */
void atari_leds_free (struct atari_leds *leds);

/*
 * Get the number of leds on a controller
 *
 * Returns: number of leds on the controller
 */
int atari_leds_get_n_leds (struct atari_leds *leds);
/*
 * Get the device type of a controller (classic or modern
 *
 * Returns: Controller type
 */
AtariDeviceType atari_leds_get_device_type (struct atari_leds *leds);

/*
 * Set a particular led to a given brightness
 * - leds: A previously created leds structure
 * - idx: The index of the LED to change (range [0 - number of leds)
 * - brightness: brightness value
 *
 * Changes to the brightness will only be applied on the next call to commit
 */
void atari_leds_set_led (struct atari_leds *leds, unsigned int idx, uint8_t brightness);

/*
 * Set a block of leds to a given brightness
 * - leds: A previously created leds structure
 * - offset: The first led to change (range [0 - number of leds)
 * - number: Number of following leds to change.
 * - brightness: brightness value to set
 *
 * Changes to the brightness will only be applied on the next call to commit.
 * This call supports wrap around; E.g. on a classic controller with 24 leds
 * using offset = 20 and number = 5 the brightness of led 20, 21, 22, 23, 0 and
 * 1 will be updated.
 */
void atari_leds_set_block (struct atari_leds *leds,
                           unsigned int offset,
                           unsigned int number,
                           uint8_t brightness);

/*
 * Set a all leds to a given brightness
 * - leds: A previously created leds structure
 * - brightness: brightness value to set
 *
 * Changes to the brightness will only be applied on the next call to commit.
 */
void atari_leds_set_all (struct atari_leds *leds, uint8_t brightness);

/*
 * Turn all leds off (brightness 0)
 * - leds: A previously created leds structure
 *
 * Changes to the brightness will only be applied on the next call to commit.
 */
void atari_leds_clear (struct atari_leds *leds);

/*
 * Commit all change to the led to the physical leds
 * - leds: A previously created leds structure
 *
 * All previous changes to the led brightness will be written out to the leds.
 */
void atari_leds_commit (struct atari_leds *leds);

/*
 * Drop application control of the LEDs
 * - leds: A previously created leds structure
 *
 * This puts the controller LEDS back under firmware control
 */
void atari_leds_drop_control (struct atari_leds *leds);

#ifdef __cplusplus
}
#endif

#endif /* atari-leds.h */
